@@ -362,6 +362,9 @@ public:
 	bool isRTL() const { return _isRTL; };
 	void setRTL(bool isRTL) { _isRTL = isRTL; };
 
+	bool isPinned() const { return _isPinned; };
+	void setPinned(bool isPinned) { _isPinned = isPinned; };
+
 private:
 	int indexOfReference(const ScintillaEditView * identifier) const;
 
@@ -445,4 +448,5 @@ private:
 	bool _isInaccessible = false;
 
 	bool _isRTL = false;
+	bool _isPinned = false;
 };
