@@ -17,7 +17,7 @@
 
 #pragma once
 
-#define DM_NOFOCUSWHILECLICKINGCAPTION TEXT("NOFOCUSWHILECLICKINGCAPTION")
+#define DM_NOFOCUSWHILECLICKINGCAPTION L"NOFOCUSWHILECLICKINGCAPTION"
 
 #define IDD_PLUGIN_DLG                  103
 #define IDC_EDIT1                       1000
